% tau(cx(t), cy(t), 1)
function [err, cpx, cpy, alpha1, beta1, b1, b2] = findCoeffs3D_T_ct_ct_1(im, time, lt1, lt2, fxx, fyy, fxy, fxt, fyt, ftt, fx, fy, ft)

NORMALIZE_SPLINES = 1;
%global fxx fyy fxy fxt fyt ftt
[pwy pwx pwt] = size(fxx);
nt1 = time/(2^lt1);
nt2 = time/(2^lt2);
M = zeros(1,nt1);
N = zeros(nt1,nt1);
T = zeros(nt2,nt1);
P = zeros(1,nt2);
R = zeros(nt1,nt2);
S = zeros(nt2,nt2);

exp_lt1 = 2^-lt1;
exp_lt2 = 2^-lt2;

[b1 hede] = meshgrid(1:time, 1:nt1);
for k=1:nt1
    tmp = 1 - abs(exp_lt1*b1(k,:)-k);
    list = find(tmp < 0);
    tmp(list) = 0;
    b1(k,:) = tmp;
end
if NORMALIZE_SPLINES
    for t=1:time
        if(sum(b1(:,t))~=1)
            b1(1,t) = 1;
        end
    end
end        

[b2 hede] = meshgrid(1:time, 1:nt2);
for k=1:nt2
    tmp = 1 - abs(exp_lt2*b2(k,:)-k);
    list = find(tmp < 0);
    tmp(list) = 0;
    b2(k,:) = tmp;
end
if NORMALIZE_SPLINES
    for t=1:time
        if(sum(b2(:,t))~=1)
            b2(1,t) = 1;
        end
    end
end        

% compute alpha
for t=1:time
    lst = find(~isnan(fxt(:,:,t)));
    sumxt = sum(sum(fxt(lst + (t-1)*pwy*pwx)));
    sumxx = sum(sum(fxx(lst + (t-1)*pwy*pwx)));
    sumxy = sum(sum(fxy(lst + (t-1)*pwy*pwx)));
    sumyy = sum(sum(fyy(lst + (t-1)*pwy*pwx)));
    sumyt = sum(sum(fyt(lst + (t-1)*pwy*pwx)));
    for ind_t=1:nt1
        M(:,ind_t) = M(:,ind_t) + b1(ind_t,t)*sumxt;
        N(:,ind_t) = N(:,ind_t) + b1(ind_t,t)*sumxx*b1(:,t);
        T(:,ind_t) = T(:,ind_t) + b1(ind_t,t)*sumxy*b2(:,t);
    end
    for ind_t=1:nt2
        P(:,ind_t) = P(:,ind_t) + b2(ind_t,t)*sumyt;
        R(:,ind_t) = R(:,ind_t) + b2(ind_t,t)*sumxy*b1(:,t);
        S(:,ind_t) = S(:,ind_t) + b2(ind_t,t)*sumyy*b2(:,t);
    end
end

X = [M P];
Q = [N R;T S];
vars = -X*Q'*pinv(Q*Q');
alpha1 = vars(1:nt1);
beta1 = vars(nt1+1:nt2+nt1);
% if QUANT
%     alpha1 = round(alpha1/0.125)*0.125;
%     beta1 = round(beta1/0.125)*0.125;
% end

cpx = zeros(time,1);
for t=1:time
    cpx(t) = alpha1*b1(:,t);
end

cpy = zeros(time,1);
for t=1:time
    cpy(t) = beta1*b2(:,t);
end

for t=1:time
    err(:,:,t) = fx(:,:,t)*cpx(t) + fy(:,:,t)*cpy(t) + ft(:,:,t);
end

err = sum(err(:).^2);